'******************************************************************************
'*  m100_video.spin - Output a 64 scanline bitmap
'*
'*  This is based on ideas and code from Debug_1pinTV by "Cluso99"
'*
'*  (c) Juergen Buchmueller <pullmoll@t-online.de>
'*
'* $Id$
'******************************************************************************
CON

#ifdef NTSC
'' NTSC 64x16
ocols			=	5			' number of "panel" columns
orows			=	64			' number of scan rows
ocolsfrq		=	22_478			' Hz per active pixel
oequal1			=	6			' number of equalization pulses
oserr			=	6			' number of serration pulses
oequal2			=	6			' number of equalization pulses
oblank1			=	32			' number of blank lines top
oactive			=	64			' number of active lines
oblank2			=	141			' number of blank lines bottom
otsync			=	39			' temp sync pulse
ohalf			=	4324			' vscl half line
obackp			=	4162	 		' vscl back porch (sync to active)
ofrontp			=	4133			' vscl front porch (active to sync)
oequal			=	4112			' vscl equalization pulse (sync/2)
osync			=	4129			' vscl sync pulse
osynch			=	4291			' vscl half-sync
oequalh			=	4308			' vscl half-equal
ovsclch1		=	4128			' vscl character (32 pixels)
ovsclch2		=	4114			' vscl character (18 pixels)
ovsclch3		=	4099			' vscl character (3 pixels)
ictra			=	11			' CTRA with PLLdiv
ifrqa			=	386169088		' required FRQA value (5MHz*16) (xfrqa replaced by start routine)
ifrqb			=	$4924_0000		' 40/140 << 32 = 1,227,096,064

ifrqa_80MHz		=	386169088		' required FRQA value (5MHz*16)
ifrqa_96MHz		=	321807584		' required FRQA value (6MHz*16)
ifrqa_100MHz		=	308935280		' required FRQA value (6.25MHz*16)
ifrqa_104MHz		=	297053152		' required FRQA value (6.5MHz*16)
ifrqa_108MHz		=	286051184		' required FRQA value (13.5MHz*8)
#endif

'--------------------------------------------------------------------------------------------------
#ifdef PAL
'' PAL	64x16 at 6x12px == 48x24 at 8x8px
ocols			=	5			' number of "panel" columns
orows			=	64			' number of scan rows
ocolsfrq		=	22_321			' Hz per active pixel
oequal1			=	4			' number of equalization pulses
oserr			=	5			' number of serration pulses
oequal2			=	5			' number of equalization pulses
oblank1			=	49			' number of blank lines top
oactive			=	64			' number of active lines
oblank2			=	175			' number of blank lines bottom
otsync			=	39			' temp sync pulse 
ohalf			=	4324			' vscl half line
obackp			=	4163			' vscl back porch (sync to active)
ofrontp			=	4132			' vscl front porch (active to sync)
oequal			=	4112			' vscl equalization pulse (sync/2)
osync			=	4129			' vscl sync pulse
osynch			=	4291			' vscl half-sync
oequalh			=	4308			' vscl half-equal
ovsclch1		=	4128			' vscl character (32 pixels)
ovsclch2		=	4114			' vscl character (18 pixels)
ovsclch3		=	4099			' vscl character (3 pixels)
ictra			=	11			' CTRA with PLLdiv
ifrqa			=	383471856		' required FRQA value (5MHz*16) (xfrqa replaced by start routine)
ifrqb			=	$4924_0000		' 40/140 << 32 = 1,227,096,064

ifrqa_80MHz		=	383471856		' required FRQA value (5MHz*16)
ifrqa_96MHz		=	319559872		' required FRQA value (6MHz*16)
ifrqa_100MHz		=	306777488		' required FRQA value (6.25MHz*16)
ifrqa_104MHz		=	294978352		' required FRQA value (6.5MHz*16)
ifrqa_108MHz		=	284053232		' required FRQA value (13.5MHz*8)
#endif
'--------------------------------------------------------------------------------------------------

io_cmd_out		=	$01
io_cmd_in		=	$02
LCD_DRIVER_W		=	50

VAR
	long	cog
	long	params[3]

PUB start(bitmap, tvPin) : okay
' bitmap is the address of the bitmap buffer * 32 + the tvPin number

	stop
	Setforxtalfreq					'set ifrqa in hub for the current clkfreq before cognew 
	params[0] := bitmap
	params[1] := tvPin
	okay := cog := COGNEW(@entry, @params) + 1

PUB stop
	if cog
		COGSTOP(cog - 1)
		cog := 0

PRI Setforxtalfreq
'' Set the frqa value depending on the actual clkfreq
	Case clkfreq
		 80_000_000: xfrqa := ifrqa_80MHz	' 5.00 MHz
		 96_000_000: xfrqa := ifrqa_96MHz	' 6.00 MHz
		100_000_000: xfrqa := ifrqa_100MHz	' 6.25 MHz
		104_000_000: xfrqa := ifrqa_104MHz	' 6.50 MHz
		108_000_000: xfrqa := ifrqa_108MHz	'13.50 MHz

DAT
		org	0
entry
		mov	ptr, PAR			' pointer to 3 longs (pio, iocmd, tvPin)
		rdlong	bitmap_ptr, ptr
		add	ptr, #4				' point to pin number
		rdlong	pin_no, ptr			' get TV pin#

		mov	FRQB, xfrqb			' default = 40/140

		movi	CTRA, #ictra
		mov	FRQA, xfrqa

		movs	CTRB, pin_no			' set pin
		mov	count, #1
		shl	count, pin_no
		mov	DIRA, count			' set pin mask
		mov	count, pin_no			' <=== can save this by next shr #4
		shr	count, #3
		movd	VCFG, count			' set VGroup
		mov	count, #1
		and	pin_no, #7			' <=== poss to save?
		shl	count, pin_no
		movs	VCFG, count			' set VPins
		movi	VCFG, #%0_01_0_0_0_000		' VGA mode, 1 bit per pixel

'**************************************************************************************************
frame
		add	framectr, #1			' inc frame counter
		mov	rows, #oequal1
		call	#equalizing			' equalisation pulses
'--------------------------------------------------------------------------------------------------
		mov	rows, #oserr			' serration pulses
:loop		mov	vscl, xsynch			' serration pulse (long)
		waitvid xFF00, #0		
		movi	CTRB, #0			' turn off blank
		mov	vscl, xsync			' serration pulse (short)
		waitvid xFF00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		djnz	rows, #:loop
'--------------------------------------------------------------------------------------------------
		mov	rows, #oequal2
		call	#equalizing			' equalisation pulses
'==================================================================================================
		mov	rows, #oblank1
		call	#doblank			' blank lines (top)
'==================================================================================================
		mov	rows, #8
		mov	bw, #%01110101
		call	#doborder			' white lines (top)
'==================================================================================================
'display active lines
		mov	bmapptr, bitmap_ptr		' initialize bitmap pointer
		mov	rows, #oactive
doactive
		mov	vscl, xsync			' 4.7uSec @ -40 IRE
		waitvid xFF00, #0
		movi	CTRB, #0			' turn off blank
		mov	vscl, xbackp			' back porch
		waitvid xFF00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		mov	count, #ocols			' "panels" per line
'--------------------------------------------------------------------------------------------------
'display entire pixel line from bmapptr..
' total of 250 + 70 = 320 pixels
		mov	vscl, xvsclch3			' 1 PLLA per pixel, 3 PLLA per frame
		mov	bits, #$02			' empty columns
		waitvid	xFF00, bits			' output to screen
		mov	vscl, xvsclch1			' 1 PLLA per pixel, 32 PLLA per frame
		test	rows, #1		WZ
	if_nz	mov	bits, pattern_0_l
	if_z	mov	bits, pattern_1_l
		waitvid	x00FF, bits			' output to screen
:active
		mov	vscl, xvsclch1			' 1 PLLA per pixel, 32 PLLA per frame
		rdlong	bits, bmapptr			' read bitmap from HUB RAM
		add	bmapptr, #4
		waitvid	x00FF, bits			' output to screen
		mov	vscl, xvsclch2			' 1 PLLA per pixel, 18 PLLA per frame
		rdlong	bits, bmapptr			' read bitmap from HUB RAM
		add	bmapptr, #4
		waitvid	x00FF, bits			' output to screen
		djnz	count, #:active			' do entire line of 250 pixels

		mov	vscl, xvsclch1			' 1 PLLA per pixel, 32 PLLA per frame
		test	rows, #1		WZ
	if_nz	mov	bits, pattern_0_r
	if_z	mov	bits, pattern_1_r
		waitvid	x00FF, bits			' output to screen
		mov	vscl, xvsclch3			' 1 PLLA per pixel, 3 PLLA per frame
		mov	bits, #$02			' empty columns
		waitvid	x00FF, bits			' output to screen
'--------------------------------------------------------------------------------------------------
		mov	vscl, xfrontp			' front porch
		waitvid xFF00, #0			' output blank
		djnz	rows, #doactive			' next row
'==================================================================================================
		mov	rows, #8
		mov	bw, #%10101110
		call	#doborder			' white lines (bottom)
'==================================================================================================
		mov	rows, #oblank2
		call	#doblank			' blank lines (bottom)
'==================================================================================================
		jmp	#frame
'**************************************************************************************************

equalizing	mov	vscl, xequal			' equalizing pulse (short)
		waitvid xFF00, #0
		movi	CTRB, #0			' turn off blank
		mov	vscl, xequalh			' equalizing pulse (long)
		waitvid xFF00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		djnz	rows, #equalizing
equalizing_ret
		ret
'--------------------------------------------------------------------------------------------------
doblank
		mov	vscl, xsync			' 4.7uSec @ -40 IRE
		waitvid xFF00, #0
		movi	CTRB, #0			' turn off blank
		mov	vscl, xsynch			' blank line
		waitvid xFF00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		mov	vscl, xhalf			' half line ???
		waitvid xFF00, #0
		djnz	rows, #doblank
doblank_ret
		ret
'--------------------------------------------------------------------------------------------------
doborder
:row
		mov	vscl, xsync			' 4.7uSec @ -40 IRE
		waitvid xFF00, #0
		movi	CTRB, #0			' turn off blank
		mov	vscl, xbackp			' back porch
		waitvid xFF00, #0
		movi	CTRB, #%0_00110_000		' turn on blank
		mov	count, #ocols			' "panels" per line

		mov	vscl, xvsclch3			' 1 PLLA per pixel, 3 PLLA per frame
		mov	bits, #$02			' empty columns
		waitvid	xFF00, bits			' output to screen
		mov	vscl, xvsclch1			' 1 PLLA per pixel, 32 PLLA per frame
		test	rows, #1		WZ
	if_nz	mov	bits, pattern_0_l
	if_z	mov	bits, pattern_1_l
		waitvid	x00FF, bits			' output to screen
:active
		test	bw, #1			WC
	if_nc	mov	bits, bits_0
	if_c	mov	bits, bits_1
		mov	vscl, xvsclch1			' 1 PLLA per pixel, 32 PLLA per frame
		waitvid	x00FF, bits			' output to screen
		mov	vscl, xvsclch2			' 1 PLLA per pixel, 18 PLLA per frame
		waitvid	x00FF, bits			' output to screen
		djnz	count, #:active			' do entire line of 250 pixels

		mov	vscl, xvsclch1			' 1 PLLA per pixel, 32 PLLA per frame
		test	rows, #1		WZ
	if_nz	mov	bits, pattern_0_r
	if_z	mov	bits, pattern_1_r
		waitvid	x00FF, bits			' output to screen
		mov	vscl, xvsclch3			' 1 PLLA per pixel, 3 PLLA per frame
		mov	bits, #$02			' empty columns
		waitvid	x00FF, bits			' output to screen

'--------------------------------------------------------------------------------------------------
		mov	vscl, xfrontp			' front porch
		waitvid xFF00, #0			' output blank
		ror	bw, #1
		djnz	rows, #:row			' next row
		rol	bw, #4
doborder_ret
		ret
'**************************************************************************************************

'--------------------------------------------------------------------------------------------------
'constants > $1FF
xhalf		long	ohalf
xbackp		long	obackp
xfrontp		long	ofrontp
xequal		long	oequal
xsync		long	osync
xsynch		long	osynch
xequalh		long	oequalh
xvsclch1	long	ovsclch1
xvsclch2	long	ovsclch2
xvsclch3	long	ovsclch3
xfrqa		long	ifrqa
xfrqb		long	ifrqb

xFF00		long	$FF00				' white / black
x00FF		long	$00FF				' black / white
low_24bits	long	$00ffffff
s40x32		long	40*32
s40x16		long	40*16
s40x8		long	40*8
pattern_0_l	long	%00110011_00110011_00110011_00110011
pattern_1_l	long	%00101100_11001100_11001100_11001100
pattern_0_r	long	%00110011_00110011_00110011_00110100
pattern_1_r	long	%11001100_11001100_11001100_11001100
bits_0		long	%00000000_00000000_00000000_00000000
bits_1		long	%11111111_11111111_11111111_11111111
bw		long	0

bitmap_ptr	long	0-0
pin_no		long	0-0

bits		res	1				' current character / character bitmap
bmapptr		res	1				' pointer to bitmap data
framectr	res	1
sync		res	1
count		res	1				' all purpose counter
rows		res	1				' row counter
ptr		res	1
t1		res	1
t2		res	1
t3		res	1
t4		res	1

		fit	$1f0

''      MIT LICENSE
{{
'  Permission is hereby granted, free of charge, to any person obtaining
'  a copy of this software and associated documentation files
'  (the "Software"), to deal in the Software without restriction,
'  including without limitation the rights to use, copy, modify, merge,
'  publish, distribute, sublicense, and/or sell copies of the Software,
'  and to permit persons to whom the Software is furnished to do so,
'  subject to the following conditions:
'
'  The above copyright notice and this permission notice shall be included
'  in all copies or substantial portions of the Software.
'
'  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
'  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
'  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
'  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
'  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
'  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
'  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
}}
